<?php
/***************************************************************
*  Copyright notice
*  
*  (c) 2004 Christian Stdemann (typo3@christian-stuedemann.de)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is 
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU GenerPublic License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
* 
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/** 
 * Plugin 'Event Table' for the 'csevents' extension.
 *
 * @author	Christian Stdemann <typo3@christian-stuedemann.de>
 * @todo	check FE-user access for categories, descriptions & appointments
 */


require_once(PATH_tslib."class.tslib_pibase.php");

class tx_csevents_pi1 extends tslib_pibase {
	var $prefixId = "tx_csevents_pi1";		// Same as class name
	var $scriptRelPath = "pi1/class.tx_csevents_pi1.php";	// Path to this script relative to the extension dir.
	var $extKey = "csevents";	// The extension key.

	var $template = '';
	var $categories;
	var $descriptions;
	var $fmtstime;
	var $fmtsdate;
	var $fmtetime;
	var $fmtletin;
	var $fmtlistdate;
	var $fmtmonth;
	var $listid;
	var $orderid;
	var $orderparams;
	var $insertOldEvents;
	var $defaultCmd;
	var $previewDays;
	var $emptyText;



	/***********************************************
	 *
	 * MAIN METHODS:
	 *
	 ***********************************************/

	/**
	 * @todo: documentation
	 */
	function main($content,$conf)	{
		$this->conf=$conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		$this->pi_USER_INT_obj=1;
		$this->init();

		$cmd = $this->piVars['cmd'];
		if (!$cmd || $this->getConf('appearance', 'forceView')) $cmd = $this->defaultCmd;

		switch($cmd) {
			case 'preview': {
				$from = time();
				$preview = $this->previewDays ? $this->previewDays : 7;
				$to = $from + ($preview*24*60*60);
				$mycontent .= $this->showList($from, $to, array());
				break;
			}
			case 'performance': {
				$id = $this->getConf('appearance', 'performance');
				$conditions['description:uid'] = $id;
				$mycontent .= $this->showList(0, 0, $conditions);
				break;
			}
			case 'months': {
				$mycontent .= $this->showMonths();
				break;
			}
			case 'list': {
				$from = $this->piVars['from'];
				$to = $this->getEnd($from);
				$mycontent .= $this->showList($from, $to, array());
				break;
			}
			case 'current': {
				$from = $this->getCurrentStart();
				$to = $this->getEnd($from);
				$mycontent .= $this->showList($from, $to, array());
				break;
			}
			case 'nearest':
			default: {
				$from = $this->getNearestStart();
				$to = $this->getEnd($from);
				$mycontent .= $this->showList($from, $to, array());
				break;
			}
		}
			
		return $content . $this->pi_wrapInBaseClass($mycontent);
	}

	/**
	 * Reads the TS configuration to initialise internal data.
	 * @return	void
	 */
	function init() {
		$this->pi_initPIflexForm(); // Init and get the flexform data of the plugin

		$this->categories = array();
		$this->descriptions = array();

		$this->template = $this->cObj->fileResource($this->getConf('appearance', 'template'));
		$this->defaultCmd = $this->getConf('appearance', 'view');
		$this->previewDays = $this->getConf('appearance', 'preview');
		$this->insertOldEvents = $this->getConf('appearance', 'oldEvents');
		$this->emptyText = $this->getConf('appearance', 'emptytext');
		$this->listid = $this->getConf('appearance', 'listuid');

		$this->fmtsdate = $this->getConf('formats', 'sdate');
		$this->fmtstime = $this->getConf('formats', 'stime');
		$this->fmtetime = $this->getConf('formats', 'etime');
		$this->fmtletin = $this->getConf('formats', 'letin');
		$this->fmtlistdate = $this->getConf('formats', 'listdate');
		$this->fmtmonth = $this->getConf('formats', 'monthlist');

		$this->orderid = $this->getConf('ordering', 'uid');
		$this->orderparams = t3lib_div::trimExplode(',', $this->getConf('ordering', 'params'), TRUE);
	}


	/***********************************************
	 *
	 * LIST OF MONTHS
	 *
	 ***********************************************/

	/**
	 * @todo: documentation
	 */
	function showMonths() {
		$content = $this->cObj->getSubpart($this->template, '###TEMPLATE_MONTHS###');
		$months = $this->collectMonths();
		$content = $this->fillMonthList($content, $months);
		return $content;
	}

	/**
	 * @todo: documentation
	 */
	function collectMonths() {
		$events = $this->getEvents(0, 0, array());
		$months = array();
		$old = 0;

		foreach (array_keys($events) as $stime) {
			$from = $this->firstOfMonth($stime);

			if($old <> $from) {
				$months[] = $from;
				$old = $from;
			}
		}

		return $months;
	}

	/**
	 * @todo: documentation
	 */
	function fillMonthList($content, $months) {
		$monthRow = $this->cObj->getSubpart($content, '###MONTH_ROW###');
		$monthsHtml = '';
		$params[$this->prefixId.'[cmd]'] = 'list';

		foreach ($months as $month) {
			$myRow = $monthRow;
			$strmonth = strftime($this->fmtmonth, $month);
			$params[$this->prefixId.'[from]'] = $month;

			$myRow = $this->wrapLink($myRow, '###LINK_TO_LIST###', $this->listid, $params);
			$myRow = $this->cObj->substituteMarker($myRow, '###MONTH###', $strmonth);
			$monthsHtml .= $myRow;
		}

		$content = $this->cObj->substituteSubpart($content, '###MONTH_ROW###', $monthsHtml);
		return $content;
	}


	/***********************************************
	 *
	 * LIST OF EVENTS (typically one month)
	 *
	 ***********************************************/

	/**
	 * Uses the TEMPLATE_LIST template to show a list of events.
	 *
	 * @param	date	$sdate : The first date/time (including) of the search period
	 * @param	date	$edate : The last date/time (excluding) of the search period
	 * @return	string	$content : The generated HTML content
	 */
	function showList($sdate, $edate, $conditions) {
		$events = $this->getEvents($sdate, $edate, $conditions);
		//$this->debug('Events', $events);
		$content = $this->fillListTemplate($events);
		$listdate = strftime($this->fmtmonth, $sdate);
		$content = $this->cObj->substituteMarker($content, '###LISTDATE###', $listdate);
		return $content;
	}

	/**
	 * Fills the TEMPLATE_LIST with data from an events array.
	 *
	 * @param	array	$events : Array of the events that shall be listed
	 * @return	string	$content : HTML formatted list of events
	 */
	function fillListTemplate($events) {
		$content = $this->cObj->getSubpart($this->template, '###TEMPLATE_LIST###');
		$rowTemplate = $this->cObj->getSubpart($content, '###EVENT_ROW###');
		$htmlRows = '';
		$odd = TRUE;

		// Loop through the whole list of event-dates
		foreach($events as $stime => $dateEvents) {
			do {
				// generate a new HTML row
				$foundEventInRow = FALSE;
				$myRowTemplate = $rowTemplate;
				$myRowTemplate = $this->cObj->substituteMarker($myRowTemplate, '###DATE###', strftime($this->fmtlistdate, $stime));
				$myRowTemplate = $this->cObj->substituteMarker($myRowTemplate, '###EVEN_ODD###', $odd ? 'odd' : 'even');

				// Loop through all events of one date (resp. all EVENT[n] subparts)
				for($i=1; TRUE; $i++) {
					$key = '###EVENT'.$i.'###';
					$eventTemplate = $this->cObj->getSubpart($myRowTemplate, $key);
					if (strlen($eventTemplate) == 0) break;
					$eventIx = $this->filterEvents($eventTemplate, $dateEvents);
					$eventTemplate = $this->cObj->substituteSubpart($eventTemplate, '###CONDITION###', '');

					if(isset($eventIx)) {
						$event = $dateEvents[$eventIx];
						$dateEvents[$eventIx]['#displayed'] = TRUE;
						$eventTemplate = $this->chooseSubpart($eventTemplate, '###EXISTS###', '###EMPTY###');
						$eventTemplate = $this->fillTemplateWithEventData($eventTemplate, $event);
						$foundEventInRow = TRUE;
					} else {
						$eventTemplate = $this->chooseSubpart($eventTemplate, '###EMPTY###', '###EXISTS###');
					}

					$myRowTemplate = $this->cObj->substituteSubpart($myRowTemplate, $key, $eventTemplate);
				}

				if ($foundEventInRow) {
					$odd = !$odd;
					$htmlRows .= $myRowTemplate;
				}
			} while ($foundEventInRow == TRUE);
		} // foreach

		// insert all rows into the list template and return the result
		if (strlen($htmlRows) == 0) {
			$content = $this->chooseSubpart($content, '###EMPTY_LIST###', '###FILLED_LIST###');
			$content = $this->cObj->substituteMarker($content, '###EMPTY_TEXT###', $this->emptyText);
		} else {
			$content = $this->chooseSubpart($content, '###FILLED_LIST###', '###EMPTY_LIST###');
			$content = $this->cObj->substituteSubpart($content, '###EVENT_ROW###', $htmlRows);
		}

		return $content;
	}

	/**
	 * @todo: documentation
	 */
	function filterEvents($eventTemplate, $dateEvents) {
		if (!is_array($dateEvents)) return NULL;
		$condition = trim($this->cObj->getSubpart($eventTemplate, '###CONDITION###'), " \t\r\n\0\x0B<!->");
		$conditions = t3lib_div::trimExplode(',', $condition, TRUE);
		$field = FALSE; $value = FALSE;

		if(is_array($conditions) && (count($conditions) == 2)) {
			$field = $conditions[0];
			$value = $conditions[1];
		}

		for ($ix = 0; $ix != count($dateEvents); $ix++) {
			$event = $dateEvents[$ix];
			if($event['#displayed']) continue;
			if(!$field) return $ix;

			switch ($field) {
				case '#categories': {
					$categories = $event['#categories'];

					foreach ($categories as $category) {
						if ($category['uid'] == $value) return $ix;
					}

					break;
				}
				case '#descr': {
					if ($event['#descr']['uid'] == $value) return $ix;
					break;
				}
				default: {
					$eventValues = $event[$field];

					if (is_array($eventValues)) {
						$eventValues = array_keys($eventValues);
					} else {
						$eventValues = t3lib_div::trimExplode(',', $eventvalues, TRUE);
					}

					if (t3lib_div::inArray($eventValues, $value)) return $ix;
					break;
				}
			}
		}

		return NULL;
	}

	/**
	 * @todo: documentation
	 */
	function fillTemplateWithEventData($eventTemplate, $event) {
		$descr = $event['#descr'];
		$pagelink = $this->getLink($descr);
		$stime = $event['stime'];
		$descrtxt = str_replace("\n", '<br>', $descr['description']);

		$eventTemplate = $this->wrapLink($eventTemplate, '###PAGELINK###', $pagelink);
		$eventTemplate = $this->cObj->substituteMarker($eventTemplate, '###STIME###', strftime($this->fmtstime, $stime));
		$eventTemplate = $this->cObj->substituteMarker($eventTemplate, '###ETIME###', strftime($this->fmtetime, $event['etime']));
		$eventTemplate = $this->cObj->substituteMarker($eventTemplate, '###LETIN###', strftime($this->fmtletin, $event['letin']));
		$eventTemplate = $this->cObj->substituteMarker($eventTemplate, '###TITLE###', $descr['title']);
		$eventTemplate = $this->cObj->substituteMarker($eventTemplate, '###DESCR###', $descrtxt);
		$eventTemplate = $this->cObj->substituteMarker($eventTemplate, '###PRICE###', $descr['price']);
		$eventTemplate = $this->cObj->substituteMarker($eventTemplate, '###REDUCED###', $descr['reduced']);

		if ($stime < time()) {
			$eventTemplate = $this->chooseSubpart($eventTemplate, '', '###DO_SOLDOUT###');
			$eventTemplate = $this->chooseSubpart($eventTemplate, '', '###DO_ORDER###');
		} else {
			if($event['soldout']) {
				// @todo: fill $soldout
				$soldout = 'Ausverkauft';
				$eventTemplate = $this->chooseSubpart($eventTemplate, '###DO_SOLDOUT###', '###DO_ORDER###');
				$eventTemplate = $this->cObj->substituteMarker($eventTemplate, '###SOLDOUT###', $soldout);
			} else {
				// @todo: fill $order
				$orderparams = $this->getOrderParameters($event);
				$order = 'Bestellen';
				$eventTemplate = $this->chooseSubpart($eventTemplate, '###DO_ORDER###', '###DO_SOLDOUT###');
				$eventTemplate = $this->wrapLink($eventTemplate, '###ORDERLINK###', $this->orderid, $orderparams);
				$eventTemplate = $this->cObj->substituteMarker($eventTemplate, '###ORDER###', $order);
			}
		}

		return $eventTemplate;
	}

	/**
	 * @todo: documentation
	 */
	function getOrderParameters($event) {
		$params = array();

		foreach ($this->orderparams as $param) {
			$evKey = ($param == 'sdate') ? 'stime' : $param;
			$urlKey = $this->prefixId . '[' . $param . ']';
			$value = $event[$evKey];
			if(strlen($value) == 0) $value = $event['#descr'][$evKey];

			switch ($param) {
				case 'stime': $value = strftime($this->fmtstime, $value); break;
				case 'sdate': $value = strftime($this->fmtsdate, $value); break;
				case 'etime': $value = strftime($this->fmtetime, $value); break;
				case 'letin': $value = strftime($this->fmtletin, $value); break;
			}

			$params[$urlKey] = $value;
		}

		return $params;
	}


	/***********************************************
	 *
	 * INTERNAL HELPER FUNCTIONS
	 *
	 ***********************************************/

	/**
	 * @todo: documentation
	 */
	function getLink($descr) {
		if($this->getConf('autolink', 'enable')) {
			$page = $descr['internal_id'];
			$wrap = $this->getConf('autolink', 'wrap');
			return $this->cObj->wrap($page, $wrap);
		} else {
			return $descr['link'];
		}
	}

	/**
	 * Chooses one of two subparts within a template. The other subpart is removed from the template.
	 *
	 * @param	string	$template : The template containing both subparts
	 * @param	string	$on : The key of the subpart that shall remain in the template
	 * @param	string	$off : The key of the subpart that shall be removed from the template
	 * @return	string	$template : The template containing the $on subpart, but not the $off subpart
	 */
	function chooseSubpart($template, $on, $off) {
		// remove the $off subpart of the template
		$template = $this->cObj->substituteSubpart($template, $off, '');

		if (strlen($on) > 0) {
			// insert the $on subpart without it's surrounding comments
			$onTemplate = $this->cObj->getSubpart($template, $on);
			$template = $this->cObj->substituteSubpart($template, $on, $onTemplate);
		}

		// return the result
		return $template;
	}

	/**
	 * Wraps the content of subpart $key in $template by $begin and $end (wrap = $begin | $end).
	 *
	 * @param	string	$template : The template containing the subpart
	 * @param	string	$key : The key of the subpart
	 * @param	string	$page : The id of the page to link to
	 * @return	string	$template : The template with a substituted / wrapped subpart
	 */
	function wrapLink($template, $key, $page, $params=array()) {
		$subpart = $this->cObj->getSubpart($template, $key);
		$subpart = $this->pi_linkToPage($subpart, $page, '', $params);
		$template = $this->cObj->substituteSubpart($template, $key, $subpart);
		return $template;
	}


	/***********************************************
	 *
	 * DATABASE QUERY FUNCTIONS
	 *
	 ***********************************************/

	/**
	 * Database query: Returns a list of all events within the period $sdate .. $edate
	 *
	 * @param	date	$sdate : The first date/time (including) of the search period
	 * @param	date	$edate : The last date/time (excluding) of the search period
	 * @param	array	$conditions : Additional conditions (field / value pairs)
	 * @return	array	$events : The list of all active events within the specified period
	 */
	function getEvents($sdate, $edate, $conditions) {
		$events = array();
		$db = $GLOBALS['TYPO3_DB'];
		$pids = $this->pi_getPidList($this->getConf('', 'pids'), $this->getConf('', 'recursive'));
		$timeCondition = $this->getTimeCondition($sdate, $edate, 'stime');
		$where = 'pid in (' . $pids . ')';
		$where .= (strlen($timeCondition) > 0) ? (' and '.$timeCondition) : '';
		$where .= ' and deleted=0 and hidden=0';
		$where .= $this->getAdditionalConditions($conditions, '');

		$res = $db->exec_SELECTquery('*', 'tx_csevents_appointments', $where, '', 'stime');

		while($event = $db->sql_fetch_assoc($res)) {
			$stime = $event['stime'];

			// add the regarding description to the record
			$descrIds = t3lib_div::intExplode(',', $event['description']);
			$descr = $this->getDescription($descrIds, $stime, $conditions);
			if (!is_array($descr)) continue;
			$event['#descr'] = $descr;

			// add the regarding categories to the record
			$categoryIds = t3lib_div::intExplode(',', $descr['category']);
			if (count($categoryIds) == 0) continue;
			$categories = array();

			foreach ($categoryIds as $catId) {
				$category = $this->getCategory($catId, $stime, $conditions);
				if (!is_array($category)) continue;
				$categories[] = $category;
			}

			if (count($categories) == 0) continue;

			$event['#categories'] = $categories;
			$id = $this->getTimeId($stime);
			$events[$id][] = $event;
		}

		$db->sql_free_result($res);

		return $events;
	}

	/**
	 * @todo: documentation
	 */
	function getTimeId($stime) {
		return $stime - ($stime % (24*60*60));	
	}

	/**
	 * @todo: documentation
	 */
	function getDescription($id, $stime, $conditions) {
		if(is_array($id)) $id = $id[0];
		if(!is_numeric($id)) return FALSE;
		$descr = $this->descriptions[$id];
		$where = $this->getAdditionalConditions($conditions, 'description');

		if (!is_array($descr)) {
			$descr = $this->readRecordFromDB('description', $id, $where);
			$this->descriptions[$id] = $descr;
		}

		return $this->checkRecordAccess($descr, $stime);
	}

	/**
	 * @todo: documentation
	 */
	function getCategory($id, $stime, $conditions) {
		if(is_array($id)) $id = $id[0];
		if(!is_numeric($id)) return FALSE;
		$category = $this->categories[$id];
		$where = $this->getAdditionalConditions($conditions, 'category');

		if (!is_array($category)) {
			// @todo: check for DB-requests that already failed
			//if ($category == FALSE) return FALSE; < doesn't work
			$category = $this->readRecordFromDB('categories', $id, $where);
			$this->categories[$id] = $category;
		}

		return $this->checkRecordAccess($category, $stime);
	}

	/**
	 * @todo: documentation
	 */
	function getAdditionalConditions($conditions, $prefix) {
		$where = '';
		
		if(is_array($conditions)) {
			foreach($conditions as $column => $value) {
				$parts = t3lib_div::trimExplode(':', $column, TRUE);
				$myPrefix = (count($parts) == 2) ? $parts[0] : '';
				
				if($myPrefix == $prefix) {
					$myColumn = (count($parts) == 2) ? $parts[1] : $column;
					$where .= ' and ' . $myColumn . '=' . $value;
				}
			}
		}

		return $where;
	}

	/**
	 * @todo: documentation
	 */
	function readRecordFromDB($table, $id, $addwhere) {
		$db = $GLOBALS['TYPO3_DB'];
		$where = 'uid='.$id.' and deleted=0 and hidden=0' . $addwhere;
		$res = $db->exec_SELECTquery('*', 'tx_csevents_'.$table, $where);
		
		if($res) {
			$record = $db->sql_fetch_assoc($res);
			$db->sql_free_result($res);
			return $record;
		} else {
			return array();
		}
	}

	/**
	 * @todo: documentation
	 */
	function checkRecordAccess($record, $stime) {
		if (!is_array($record)) return FALSE;

		if ($stime > 0) {
			if (($record['starttime'] > 0) && ($stime < $record['starttime'])) return FALSE;
			if (($record['endtime'] > 0) && ($stime > $record['endtime'])) return FALSE;
		}

		return $record;
	}

	/**
	 * Convert $sdate and $edate into a SQL WHERE-clause for a time period
	 *
	 * @param	date	$sdate : The first date/time (including) of the search period
	 * @param	date	$edate : The last date/time (excluding) of the search period
	 * @param	string	$column : The name of the DB column that holds the time information
	 * @return	string	$where : The WHERE-clause to select the correct period of time
	 */
	function getTimeCondition($sdate, $edate, $column) {
		if (!$this->insertOldEvents) {
			$now = time();
			if ($now > $sdate) $sdate = $now;
		}

		$scond = $sdate ? ($column . '>=' . $sdate) : '';
		$econd = $edate ? ($column . '<' . $edate) : '';
		return $scond . (((strlen($scond) > 0) && (strlen($econd) > 0)) ? ' and ' : '') . $econd;
	}

	/**
	 * @todo: documentation
	 */
	function getConf($section, $key) {
		$ffpage = $section ? $section : 'sDEF';
		$result = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], $key, $ffpage);

		if(!$result) {
			$array = $section ? $this->conf[$section.'.'] : $this->conf;

			if(is_array($array)) {
				$result = $array[$key];
			}
		}

		return $result;
	}

	/**
	 * @todo: documentation
	 */
	function getCurrentStart() {
		return $this->firstOfMonth(time());
	}

	/**
	 * @todo: documentation
	 */
	function getNearestStart() {
		$start = time();
		$events = $this->getEvents($start, 0, FALSE);

		if(is_array($events)) {
			$dates = array_keys($events);
			$start = $dates[0];
		}

		return $this->firstOfMonth($start);
	}

	/**
	 * @todo: documentation
	 */
	function getEnd($start) {
		$current = getdate($start);
		return mktime(0, 0, 0, $current['mon']+1, 1, $current['year']);
	}

	/**
	 * @todo: documentation
	 */
	function firstOfMonth($timestamp) {
		$date = getdate($timestamp);
		return mktime(0, 0, 0, $date['mon'], 1, $date['year']);
	}

	/**
	 * @todo: documentation
	 */
	function debug($where, $what) {
		print '<table><tr><td>'.$where.'</td><td>';
		t3lib_div::debug($what);
		print '</td></tr></table>';
	}
}



if (defined("TYPO3_MODE") && $TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/csevents/pi1/class.tx_csevents_pi1.php"])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/csevents/pi1/class.tx_csevents_pi1.php"]);
}

?>
